import logging

from PySide6.QtWidgets import QWidget, QVBoxLayout, QPushButton, QLabel


logger = logging.getLogger(__name__)
class RunButtonWidget(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent=parent)

        self._layout = QVBoxLayout()

        self._title = QLabel(f"Detect skeleton in image:")
        self._layout.addWidget(self._title)
        
        self.run_button_widget = QPushButton('Run',self)
        self._layout.addWidget(self.run_button_widget)

        self.setLayout(self._layout)



import logging
import cv2

logger = logging.getLogger(__name__)


# Constants for key actions
KEY_INCREASE_EXPOSURE = ord("w")
KEY_DECREASE_EXPOSURE = ord("s")
KEY_RESET_EXPOSURE = ord("r")
KEY_QUIT = ord("q")


class WebcamDemoViewer:
    DEFAULT_EXPOSURE = -7

    def __init__(
        self,
        tracker,
        recorder=None,
        window_title: str = None,
        default_exposure: int = DEFAULT_EXPOSURE,
    ):
        """
        Initialize with a tracker and optional window title and default exposure.
        """
        self.tracker = tracker
        self.recorder = recorder
        self.default_exposure = default_exposure
        if window_title is None:
            window_title = f"{tracker.__class__.__name__}"
        self.window_title = window_title

    def _set_exposure(self, cap, exposure):
        """
        Set the exposure of the camera.
        """
        cap.set(cv2.CAP_PROP_EXPOSURE, exposure)

    def _show_overlay(self, image, text):
        """
        Overlay text on the image.
        """
        y0, dy = 30, 25  # y0 - initial y value, dy - offset between lines
        for i, line in enumerate(text.split("\n")):
            y = y0 + i * dy
            cv2.putText(
                image, line, (10, y), cv2.FONT_HERSHEY_SIMPLEX, 0.75, (100, 0, 255), 2
            )

    def run(self):
        """
        Run the camera viewer.
        """
        cap = cv2.VideoCapture(0)
        if not cap.isOpened():
            logger.error("Error: Could not open camera.")
            return

        exposure = self.default_exposure
        self._set_exposure(cap, exposure)

        while True:
            ret, frame = cap.read()

            if not ret:
                logger.error("Error: Failed to read frame.")
                break

            image_size = (frame.shape[1], frame.shape[0])

            self.tracker.process_image(frame)
            annotated_image = self.tracker.annotated_image
            if self.recorder is not None:
                self.recorder.record(tracked_objects=self.tracker.tracked_objects)

            key = cv2.waitKey(1) & 0xFF
            if key == KEY_QUIT:
                break
            elif key == KEY_INCREASE_EXPOSURE:
                exposure += 1
                self._set_exposure(cap, exposure)
            elif key == KEY_DECREASE_EXPOSURE:
                exposure -= 1
                self._set_exposure(cap, exposure)
            elif key == KEY_RESET_EXPOSURE:
                exposure = self.default_exposure
                self._set_exposure(cap, exposure)

            self._show_overlay(
                annotated_image,
                f"Exposure: {exposure}\n"
                f"Controls: \n`w`/`s`: exposure +/- \n'r': reset \n'q': quit",
            )
            cv2.imshow(self.window_title, annotated_image)

        cap.release()
        cv2.destroyAllWindows()

        if self.recorder is not None:
            self.recorder.process_tracked_objects(image_size=image_size)
            self.recorder.save("recorded_objects.npy")



import logging
from multiprocessing import Pool, cpu_count
from pathlib import Path
import sys
from typing import Any, Optional, Type
import numpy as np
from pydantic import BaseModel


from skellytracker.trackers.base_tracker.base_tracker import BaseTracker
from skellytracker.trackers.bright_point_tracker.brightest_point_tracker import (
    BrightestPointTracker,
)
from skellytracker.trackers.mediapipe_tracker.mediapipe_holistic_tracker import (
    MediapipeHolisticTracker,
)
from skellytracker.trackers.yolo_mediapipe_combo_tracker.yolo_mediapipe_combo_tracker import (
    YOLOMediapipeComboTracker,
)
from skellytracker.trackers.yolo_tracker.yolo_tracker import YOLOPoseTracker
from skellytracker.trackers.mediapipe_tracker.mediapipe_model_info import (
    MediapipeTrackingParams,
)

logger = logging.getLogger(__name__)

file_name_dictionary = {
    "MediapipeHolisticTracker": "mediapipe2dData_numCams_numFrames_numTrackedPoints_pixelXY.npy",
    "YOLOMediapipeComboTracker": "mediapipe2dData_numCams_numFrames_numTrackedPoints_pixelXY.npy",
    "YOLOPoseTracker": "yolo2dData_numCams_numFrames_numTrackedPoints_pixelXY.npy",
    "BrightestPointTracker": "brightestPoint2dData_numCams_numFrames_numTrackedPoints_pixelXY.npy",
}


def process_folder_of_videos(
    tracker_name: str,
    tracking_params: BaseModel,
    synchronized_video_path: Path,
    output_folder_path: Optional[Path] = None,
    annotated_video_path: Optional[Path] = None,
    num_processes: int = None,
) -> None:
    """
    Process a folder of synchronized videos with the given tracker.
    Tracked data will be saved to a .npy file with the shape (numCams, numFrames, numTrackedPoints, pixelXYZ).

    :param tracker_name: Tracker to use.
    :param tracking_params: Tracking parameters to use.
    :param synchronized_video_path: Path to folder of synchronized videos.
    :param output_folder_path: Path to save tracked data to.
    :param annotated_video_path: Path to save annotated videos to.
    :param num_processes: Number of processes to use, 1 to disable multiprocessing.
    :return: Array of tracking data
    """
    if num_processes is None:
        num_processes = min(
            (cpu_count() - 1), len(list(synchronized_video_path.glob("*.mp4")))
        )

    file_name = file_name_dictionary[tracker_name]
    synchronized_video_path = Path(synchronized_video_path)
    if output_folder_path is None:
        output_folder_path = (
            synchronized_video_path.parent / "output_data" / "raw_data" / file_name
        )
    else:
        output_folder_path = Path(output_folder_path) / file_name
    if not output_folder_path.exists():
        output_folder_path.parent.mkdir(parents=True, exist_ok=True)

    if annotated_video_path is None:
        annotated_video_path = synchronized_video_path.parent / "annotated_videos"
    if not annotated_video_path.exists():
        annotated_video_path.mkdir(parents=True, exist_ok=True)

    tasks = [
        (tracker_name, tracking_params, video_path, annotated_video_path)
        for video_path in synchronized_video_path.glob("*.mp4")
    ]

    if num_processes > 1:
        logging.info("Using multiprocessing to run pose estimation")
        with Pool(processes=num_processes) as pool:
            array_list = pool.starmap(process_single_video, tasks)
    else:
        array_list = []
        for task in tasks:
            array_list.append(process_single_video(*task))

    combined_array = np.stack(array_list)

    logger.info(f"Shape of output array: {combined_array.shape}")
    np.save(output_folder_path, combined_array)

    return combined_array


def process_single_video(
    tracker_name: str,
    tracking_params: BaseModel,
    video_path: Path,
    annotated_video_path: Path,
) -> np.ndarray:
    """
    Process a single video with the given tracker.
    Tracked data will be saved to a .npy file with the shape (numCams, numFrames, numTrackedPoints, pixelXYZ).

    :param tracker_name: Tracker to use.
    :param tracking_params: Tracking parameters to use.
    :param video_path: Path to video.
    :param annotated_video_path: Path to save annotated video to.
    :return: Array of tracking data
    """
    video_name = (
        video_path.stem + "_mediapipe.mp4"
    )  # TODO: fix it so blender output doesn't require mediapipe addendum here
    tracker = get_tracker(tracker_name=tracker_name, tracking_params=tracking_params)
    logger.info(
        f"Processing video: {video_name} with tracker: {tracker.__class__.__name__}"
    )
    output_array = tracker.process_video(
        input_video_filepath=video_path,
        output_video_filepath=annotated_video_path / video_name,
        save_data_bool=False,
    )
    return output_array


def get_tracker(tracker_name: str, tracking_params: BaseModel) -> BaseTracker:
    """
    Returns a tracker object based on the given tracker_type and tracking_params.

    :param tracker_type (str): The type of tracker to be created.
    :param tracking_params (BaseModel): The tracking parameters to be used for creating the tracker.
    :return BaseTracker: The tracker object based on the given tracker_type and tracking_params.
    :raise ValueError: If an invalid tracker_type is provided.
    """
    match tracker_name:
        case "MediapipeHolisticTracker":
            tracker = MediapipeHolisticTracker(
                model_complexity=tracking_params.mediapipe_model_complexity,
                min_detection_confidence=tracking_params.min_detection_confidence,
                min_tracking_confidence=tracking_params.min_tracking_confidence,
                static_image_mode=tracking_params.static_image_mode,
            )

        case "YOLOMediapipeComboTracker":
            tracker = YOLOMediapipeComboTracker(
                model_complexity=tracking_params.mediapipe_model_complexity,
                min_detection_confidence=tracking_params.min_detection_confidence,
                min_tracking_confidence=tracking_params.min_tracking_confidence,
                static_image_mode=True,  # yolo cropping must be run with static image mode due to changing size of bounding boxes
            )

        case "YOLOPoseTracker":
            tracker = YOLOPoseTracker(
                model_size="medium",
            )

        case "BrightestPointTracker":
            tracker = BrightestPointTracker()

        case _:
            raise ValueError("Invalid tracker type")

    return tracker


if __name__ == "__main__":
    synchronized_video_path = Path(
        "/Users/philipqueen/freemocap_data/recording_sessions/freemocap_sample_data/synchronized_videos"
    )
    tracker_name = "YOLOMediapipeComboTracker"
    num_processes = None

    process_folder_of_videos(
        tracker_name=tracker_name,
        tracking_params=MediapipeTrackingParams(),
        synchronized_video_path=synchronized_video_path,
        num_processes=num_processes,
    )



import os
from datetime import datetime
from pathlib import Path
from typing import List


class Quine:
    """
    A class to represent a Quine.

    A Quine is a computer program which takes no input and produces a copy of its own source code as its only output.
    In this case, the term "Quine" is used a bit more loosely to represent a program that prints out the source code
    in a directory. This can be useful for various purposes, such as feeding the code through a Language-to-Language
    Migration (LLM) tool.

    ...

    Attributes
    ----------
    base_directory : str
        the base directory to start from
    excluded_directories : set
        a set of directories to be excluded
    included_extensions : set
        a set of file extensions to be excluded

    Methods
    -------
    write_to_file(root_directory:str, file_name:str, output_file:object):
        Writes the content of the file to the output markdown file.
    generate_quine():
        Walks through the base directory and its subdirectories, excluding certain directories and file types,
        and writes the Python source code to a Markdown file with headings for each folder and subheadings for each file.
        The source code is enclosed in ```python ``` code blocks.
    """

    def __init__(self, base_directory: str, excluded_directories: List[str], included_extensions: List[str]):
        self.base_directory = base_directory
        self.excluded_directories = excluded_directories
        self.included_extensions = included_extensions

    def write_to_file(self, root_directory: str, file_name: str, output_file: object) -> None:
        """
        Writes the content of the file to the output markdown file.

        Parameters
        ----------
        root_directory : str
            The root directory.
        file_name : str
            The file name.
        output_file : object
            The output file object.
        """
        output_file.write(f"## {file_name}\n\n")
        output_file.write("```python\n")
        with open(os.path.join(root_directory, file_name), "r") as file:
            file_content = file.read()
            output_file.write(file_content)
            print(file_content)  # Print the file content to the terminal
        output_file.write("\n```\n\n")

    def generate_quine(self) -> None:
        """
        Walks through the base directory and its subdirectories, excluding certain directories and file types,
        and writes the Python source code to a Markdown file with headings for each folder and subheadings for each file.
        The source code is enclosed in ```python ``` code blocks.
        """
        current_time = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
        file_name = f"quine_{current_time}.md"
        output_dir = Path().cwd() / "output"  # Output directory
        output_dir.mkdir(parents=True, exist_ok=True)  # Create the output directory if it doesn't exist
        file_path = output_dir / file_name  # Output file path
        with open(file_path, "w") as output_file:
            for root_directory, directories, files in os.walk(self.base_directory):
                directories[:] = [directory for directory in directories if directory not in self.excluded_directories]
                if root_directory != ".":
                    output_file.write(f"# {os.path.relpath(root_directory, '..')}\n\n")
                for file_name in files:
                    if any(file_name.endswith(extension) for extension in self.included_extensions):
                        self.write_to_file(root_directory, file_name, output_file)


if __name__ == "__main__":
    # base_directory_in =r"C:\Users\jonma\github_repos\freemocap_organization\skelly_tracker\skelly_tracker\trackers"
    base_directory_in = r"C:\Users\jonma\github_repos\freemocap_organization\skelly_tracker\skelly_tracker\trackers"
    quine = Quine(
        base_directory=base_directory_in,
        excluded_directories=["__pycache__",
                              ".git",
                              "output",
                              # "mediapipe_tracker",
                              # "charuco_tracker",
                              "mmpose_tracker"],
        included_extensions=[".py"])
    quine.generate_quine()



import logging

from PySide6.QtCore import Qt
from PySide6.QtWidgets import (
    QApplication,
    QWidget,
    QVBoxLayout,
    QPushButton,
    QFileDialog,
    QMainWindow,
    QLabel,
)

from skellytracker.gui.widgets.run_button_widget import RunButtonWidget


logger = logging.getLogger(__name__)


class MainWindow(QMainWindow):
    def __init__(self):
        logger.info("Initializing the main window")
        super().__init__()

        self.setGeometry(100, 100, 600, 600)

        widget = QWidget()
        self._layout = QVBoxLayout()
        self._layout.setAlignment(Qt.AlignmentFlag.AlignCenter)
        widget.setLayout(self._layout)
        self.setCentralWidget(widget)

        self.folder_open_button = QPushButton("Load an image")
        self._layout.addWidget(self.folder_open_button)
        self.folder_open_button.clicked.connect(self._open_session_folder_dialog)

        self._path_to_folder_label = QLabel("No image file selected")
        self._layout.addWidget(self._path_to_folder_label)

        self.run_button = RunButtonWidget(self)
        self._layout.addWidget(self.run_button)
        self.run_button.run_button_widget.clicked.connect(
            lambda: print("Run button clicked")
        )

    def _open_session_folder_dialog(self):
        file_types = "Image files (*.jpg *.jpeg *.png)"
        self._image_path, _ = QFileDialog.getOpenFileName(
            None, "Choose an image", "", file_types
        )
        self._path_to_folder_label.setText(self._image_path)


def run_gui_window():
    app = QApplication([])
    win = MainWindow()
    win.show()
    app.exec()


if __name__ == "__main__":
    run_gui_window()



import cv2
from pathlib import Path

from skellytracker.trackers.bright_point_tracker.brightest_point_tracker import BrightestPointTracker
from skellytracker.trackers.charuco_tracker.charuco_tracker import CharucoTracker
from skellytracker.trackers.mediapipe_tracker.mediapipe_holistic_tracker import MediapipeHolisticTracker
from skellytracker.trackers.yolo_tracker.yolo_tracker import YOLOPoseTracker

if __name__ == "__main__":
    demo_tracker = "brightest_point_tracker"
    image_path = Path("/Path/To/Your/Image.jpg")

    if demo_tracker == "brightest_point_tracker":
        BrightestPointTracker().image_demo(image_path=image_path)

    elif demo_tracker == "charuco_tracker":
        CharucoTracker(squaresX=7,
                       squaresY=5,
                       dictionary=cv2.aruco.getPredefinedDictionary(cv2.aruco.DICT_4X4_250)).image_demo(image_path=image_path)

    elif demo_tracker == "mediapipe_holistic_tracker":
        MediapipeHolisticTracker(model_complexity=2,
                                 min_detection_confidence=0.5,
                                 min_tracking_confidence=0.5,
                                 static_image_mode=False,
                                 smooth_landmarks=True).image_demo(image_path=image_path)

    elif demo_tracker == "yolo_tracker":
        YOLOPoseTracker(model_size="high_res").image_demo(image_path=image_path)


from datetime import datetime
import time
from pathlib import Path

from skellytracker import __package_name__

BASE_FOLDER_NAME = f"{__package_name__}_data"
LOGS_INFO_AND_SETTINGS_FOLDER_NAME = "logs_info_and_settings"
LOG_FILE_FOLDER_NAME = "logs"

def get_base_folder_path():
    base_folder =  Path().home() / BASE_FOLDER_NAME
    base_folder.mkdir(exist_ok=True, parents=True)
    return base_folder

def get_log_file_path():
    log_file_path = get_base_folder_path() / LOGS_INFO_AND_SETTINGS_FOLDER_NAME / LOG_FILE_FOLDER_NAME / create_log_file_name()
    log_file_path.parent.mkdir(exist_ok=True, parents=True)
    return log_file_path

def create_log_file_name():
    return "log_" + get_iso6201_time_string() + ".log"


def get_gmt_offset_string():
    # from - https://stackoverflow.com/a/53860920/14662833
    gmt_offset_int = int(time.localtime().tm_gmtoff / 60 / 60)
    return f"{gmt_offset_int:+}"

def get_iso6201_time_string(timespec: str = "milliseconds", make_filename_friendly: bool = True):
    iso6201_timestamp = datetime.now().isoformat(timespec=timespec)
    gmt_offset_string = f"_gmt{get_gmt_offset_string()}"
    iso6201_timestamp_w_gmt = iso6201_timestamp + gmt_offset_string
    if make_filename_friendly:
        iso6201_timestamp_w_gmt = iso6201_timestamp_w_gmt.replace(":", "_")
        iso6201_timestamp_w_gmt = iso6201_timestamp_w_gmt.replace(".", "ms")
    return iso6201_timestamp_w_gmt


from mediapipe.python.solutions import holistic as mp_holistic
from mediapipe.python.solutions.face_mesh import FACEMESH_NUM_LANDMARKS_WITH_IRISES

from skellytracker.trackers.base_tracker.base_tracking_params import BaseTrackingParams

mediapipe_body_landmark_names = [
    landmark.name.lower() for landmark in mp_holistic.PoseLandmark
]
mediapipe_hand_landmark_names = [
    landmark.name.lower() for landmark in mp_holistic.HandLandmark
]


class MediapipeModelInfo:
    num_tracked_points_body = len(mediapipe_body_landmark_names)
    num_tracked_points_face = FACEMESH_NUM_LANDMARKS_WITH_IRISES
    num_tracked_points_left_hand = len(mediapipe_hand_landmark_names)
    num_tracked_points_right_hand = len(mediapipe_hand_landmark_names)
    num_tracked_points_total = (
        len(mediapipe_body_landmark_names)
        + 2 * len(mediapipe_hand_landmark_names)
        + FACEMESH_NUM_LANDMARKS_WITH_IRISES
    )
    mediapipe_tracked_object_names = [
        "pose_landmarks",
        "face_landmarks",
        "left_hand_landmarks",
        "right_hand_landmarks",
    ]


class MediapipeTrackingParams(BaseTrackingParams):
    mediapipe_model_complexity: int = 1
    min_detection_confidence: float = 0.5
    min_tracking_confidence: float = 0.5
    static_image_mode: bool = True



import logging
from typing import Dict

import cv2
import numpy as np

from skellytracker.trackers.base_tracker.base_tracker import BaseTracker
from skellytracker.trackers.base_tracker.tracked_object import TrackedObject
from skellytracker.trackers.bright_point_tracker.brightest_point_recorder import BrightestPointRecorder

UPPER_BOUND_COLOR = [255, 255, 255]

LOWER_BOUND_COLOR = [40, 40, 40]

logger = logging.getLogger(__name__)


class BrightestPointTracker(BaseTracker):
    luminance_threshold: int = 200

    def __init__(self):
        super().__init__(tracked_object_names=["brightest_point"], recorder=BrightestPointRecorder())

    def process_image(self, image: np.ndarray, **kwargs) -> Dict[str, TrackedObject]:
        # Convert the image to grayscale
        gray_image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

        # Threshold the image to get only bright regions
        _, thresholded_image = cv2.threshold(gray_image, self.luminance_threshold, 255, cv2.THRESH_BINARY)

        # Find contours of the bright regions
        bright_patches, _ = cv2.findContours(thresholded_image, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)

        # Process each bright patch separately
        largest_area = 0
        largest_patch_centroid = None
        for patch in bright_patches:
            # Calculate the centroid of the bright patch
            patch_moments = cv2.moments(patch)
            if patch_moments["m00"] != 0:  # Avoid division by zero
                centroid_x = int(patch_moments["m10"] / patch_moments["m00"])
                centroid_y = int(patch_moments["m01"] / patch_moments["m00"])

                # Keep track of the largest patch and its centroid
                area = cv2.contourArea(patch)
                if area > largest_area:
                    largest_area = area
                    largest_patch_centroid = (centroid_x, centroid_y)

        # If a largest patch was found, update the tracked object
        if largest_patch_centroid is not None:
            self.tracked_objects["brightest_point"].pixel_x = largest_patch_centroid[0]
            self.tracked_objects["brightest_point"].pixel_y = largest_patch_centroid[1]
            self.tracked_objects["brightest_point"].extra["thresholded_image"] = thresholded_image

        self.raw_image = image.copy()

        self.annotated_image = self.annotate_image(image=image,
                                                   tracked_objects=self.tracked_objects)

        return self.tracked_objects

    def annotate_image(self, image: np.ndarray, tracked_objects: Dict[str, TrackedObject], **kwargs) -> np.ndarray:
        # Copy the original image for annotation
        annotated_image = image.copy()

        # Draw a red 'X' over the largest bright patch
        if tracked_objects["brightest_point"].pixel_x is not None and tracked_objects[
            "brightest_point"].pixel_y is not None:
            cv2.drawMarker(annotated_image,
                           (tracked_objects["brightest_point"].pixel_x, tracked_objects["brightest_point"].pixel_y),
                           (0, 0, 255), markerType=cv2.MARKER_CROSS, markerSize=20, thickness=2)

        return annotated_image


if __name__ == "__main__":
    BrightestPointTracker().demo()



from abc import ABC, abstractmethod
import logging
from typing import Dict

import numpy as np

from skellytracker.trackers.base_tracker.tracked_object import TrackedObject

logger = logging.getLogger(__name__)


class BaseRecorder(ABC):
    """
    An abstract base class for implementing different recording algorithms.
    """

    def __init__(self):
        self.recorded_objects = []
        self.recorded_objects_array = None

    @abstractmethod
    def record(
        self, tracked_objects: Dict[str, TrackedObject], annotated_image: np.ndarray
    ) -> None:
        """
        Record the tracked objects as they are created by the tracker.

        :param tracked_object: A tracked objects dictionary.
        :param annotated_image: Image array with tracking results annotated.
        :return: None
        """
        pass

    @abstractmethod
    def process_tracked_objects(self, **kwargs) -> np.ndarray:
        """
        Process the recorded objects to be in a consistent array format across trackers.

        :return: Array of tracked objects.
        """
        pass

    def clear_recorded_objects(self):
        logger.info("Clearing recorded objects from recorder")
        self.recorded_objects = []
        self.recorded_objects_array = None

    def save(self, file_path: str) -> None:
        """
        Save the recorded objects to a file.

        :param file_path: The path to the file where the recorded objects should be saved.
        :return: None
        """
        if self.recorded_objects_array is None:
            self.process_tracked_objects()
        logger.info(f"Saving recorded objects to {file_path}")
        np.save(file_path, self.recorded_objects_array)



def returnTrue(num):
    try:
        return True
    except:
        return False


def test_test():
    """This is a test of the test framework.  It should always pass.
    To make your own tests, copy this function, change the name, and add your own assertions.
    """
    assert returnTrue(6) == True


# __main__.py
import sys
from pathlib import Path

base_package_path = Path(__file__).parent.parent
print(f"adding base_package_path: {base_package_path} : to sys.path")
sys.path.insert(0, str(base_package_path))  # add parent directory to sys.path

import logging

logger = logging.getLogger(__name__)

from gui.gui import run_gui_window

def main():
    run_gui_window()

if __name__ == '__main__':
    logger.info(f"Running as a script")
    main()



from dataclasses import dataclass, field
from typing import Any, Dict, Optional


@dataclass
class TrackedObject:
    """
    A dataclass for storing information about a tracked object in a single image/frame
    """
    object_id: str
    pixel_x: Optional[float] = None
    pixel_y: Optional[float] = None
    depth_z: Optional[float] = None
    extra: Optional[Dict[str, Any]] = field(default_factory=dict)


import cv2
import numpy as np
import mediapipe as mp
from typing import Dict
from ultralytics import YOLO

from skellytracker.trackers.base_tracker.base_tracker import BaseTracker
from skellytracker.trackers.base_tracker.tracked_object import TrackedObject
from skellytracker.trackers.mediapipe_tracker.mediapipe_holistic_recorder import (
    MediapipeHolisticRecorder,
)
from skellytracker.trackers.mediapipe_tracker.mediapipe_model_info import (
    MediapipeModelInfo,
)
from skellytracker.trackers.yolo_object_tracker.yolo_object_model_info import (
    yolo_object_model_dictionary,
)


class YOLOMediapipeComboTracker(BaseTracker):
    def __init__(
        self,
        model_size: str = "nano",
        model_complexity=2,
        min_detection_confidence=0.5,
        min_tracking_confidence=0.5,
        static_image_mode=False,
        smooth_landmarks=True,
    ):
        super().__init__(
            tracked_object_names=MediapipeModelInfo.mediapipe_tracked_object_names,
            recorder=MediapipeHolisticRecorder(),
        )
        self.mp_drawing = mp.solutions.drawing_utils
        self.mp_holistic = mp.solutions.holistic
        self.holistic = self.mp_holistic.Holistic(
            model_complexity=model_complexity,
            min_detection_confidence=min_detection_confidence,
            min_tracking_confidence=min_tracking_confidence,
            static_image_mode=static_image_mode,
            smooth_landmarks=smooth_landmarks,
        )

        pytorch_model = yolo_object_model_dictionary[model_size]
        self.model = YOLO(pytorch_model)

    def process_image(self, image: np.ndarray, **kwargs) -> Dict[str, TrackedObject]:
        yolo_results = self.model(image, classes=0, max_det=1, verbose=False)

        box_xyxy = np.asarray(yolo_results[0].boxes.xyxy).flatten()

        if box_xyxy.size > 0:
            box_left, box_top, box_right, box_bottom = box_xyxy
            cropped_image = image[
                int(box_top) : int(box_bottom),
                int(box_left) : int(box_right),
            ]
        else:
            # eventually we should not even run mediapipe if no bbox is found
            box_left, box_top, box_right, box_bottom = 0, 0, 0, 0
            cropped_image = image

        cropped_rgb_image = cv2.cvtColor(cropped_image, cv2.COLOR_BGR2RGB)

        mediapipe_results = self.holistic.process(cropped_rgb_image)

        # Update the tracking data
        self._rescale_cropped_data(
            image, box_left, box_top, box_right, box_bottom, mediapipe_results
        )
        self.tracked_objects["pose_landmarks"].extra[
            "landmarks"
        ] = mediapipe_results.pose_landmarks
        self.tracked_objects["face_landmarks"].extra[
            "landmarks"
        ] = mediapipe_results.face_landmarks
        self.tracked_objects["left_hand_landmarks"].extra[
            "landmarks"
        ] = mediapipe_results.left_hand_landmarks
        self.tracked_objects["right_hand_landmarks"].extra[
            "landmarks"
        ] = mediapipe_results.right_hand_landmarks

        bbox_image = yolo_results[0].plot()

        self.annotated_image = self.annotate_image(
            image=bbox_image, tracked_objects=self.tracked_objects
        )

        return self.tracked_objects

    def _rescale_cropped_data(
        self,
        image: np.ndarray,
        box_left: int,
        box_top: int,
        box_right: int,
        box_bottom: int,
        mediapipe_results,
    ) -> None:
        if mediapipe_results.pose_landmarks is not None:
            for landmark in mediapipe_results.pose_landmarks.landmark:
                landmark.x = (
                    (landmark.x * (box_right - box_left)) + box_left
                ) / image.shape[1]
                landmark.y = (
                    (landmark.y * (box_bottom - box_top)) + box_top
                ) / image.shape[0]
                landmark.z = (landmark.z * (box_right - box_left)) / image.shape[1]
        if mediapipe_results.face_landmarks is not None:
            for landmark in mediapipe_results.face_landmarks.landmark:
                landmark.x = (
                    (landmark.x * (box_right - box_left)) + box_left
                ) / image.shape[1]
                landmark.y = (
                    (landmark.y * (box_bottom - box_top)) + box_top
                ) / image.shape[0]
                landmark.z = (landmark.z * (box_right - box_left)) / image.shape[1]
        if mediapipe_results.left_hand_landmarks is not None:
            for landmark in mediapipe_results.left_hand_landmarks.landmark:
                landmark.x = (
                    (landmark.x * (box_right - box_left)) + box_left
                ) / image.shape[1]
                landmark.y = (
                    (landmark.y * (box_bottom - box_top)) + box_top
                ) / image.shape[0]
                landmark.z = (landmark.z * (box_right - box_left)) / image.shape[1]
        if mediapipe_results.right_hand_landmarks is not None:
            for landmark in mediapipe_results.right_hand_landmarks.landmark:
                landmark.x = (
                    (landmark.x * (box_right - box_left)) + box_left
                ) / image.shape[1]
                landmark.y = (
                    (landmark.y * (box_bottom - box_top)) + box_top
                ) / image.shape[0]
                landmark.z = (landmark.z * (box_right - box_left)) / image.shape[1]

    def annotate_image(
        self, image: np.ndarray, tracked_objects: Dict[str, TrackedObject], **kwargs
    ) -> np.ndarray:
        # Draw the pose, face, and hand landmarks on the image
        self.mp_drawing.draw_landmarks(
            image,
            tracked_objects["pose_landmarks"].extra["landmarks"],
            self.mp_holistic.POSE_CONNECTIONS,
        )
        self.mp_drawing.draw_landmarks(
            image,
            tracked_objects["face_landmarks"].extra["landmarks"],
            self.mp_holistic.FACEMESH_TESSELATION,
        )
        self.mp_drawing.draw_landmarks(
            image,
            tracked_objects["left_hand_landmarks"].extra["landmarks"],
            self.mp_holistic.HAND_CONNECTIONS,
        )
        self.mp_drawing.draw_landmarks(
            image,
            tracked_objects["right_hand_landmarks"].extra["landmarks"],
            self.mp_holistic.HAND_CONNECTIONS,
        )

        return image


if __name__ == "__main__":
    YOLOMediapipeComboTracker().demo()
    # image_path = Path("/Users/philipqueen/Downloads/DSC07372.jpg")
    # YOLOMediapipeComboTracker().image_demo(image_path=image_path)



import numpy as np
from typing import Dict
from ultralytics import YOLO

from skellytracker.trackers.base_tracker.base_tracker import BaseTracker
from skellytracker.trackers.base_tracker.tracked_object import TrackedObject
from skellytracker.trackers.yolo_tracker.yolo_model_info import YOLOModelInfo
from skellytracker.trackers.yolo_tracker.yolo_recorder import YOLORecorder


class YOLOPoseTracker(BaseTracker):
    def __init__(self, model_size: str = "nano"):
        super().__init__(tracked_object_names=[], recorder=YOLORecorder())

        pytorch_model = YOLOModelInfo.model_dictionary[model_size]
        self.model = YOLO(pytorch_model)

    def process_image(self, image: np.ndarray, **kwargs) -> Dict[str, TrackedObject]:
        # "max_det=1" argument to limit to single person tracking for now
        results = self.model(image, max_det=1, verbose=False)

        self.unpack_results(results)

        self.annotated_image = self.annotate_image(image, results=results, **kwargs)

        return self.tracked_objects

    def annotate_image(self, image: np.ndarray, results, **kwargs) -> np.ndarray:
        return results[-1].plot()

    def unpack_results(self, results):
        tracked_person = np.asarray(results[-1].keypoints.xy)
        self.tracked_objects["tracked_person"] = TrackedObject(
            object_id="tracked_person"
        )
        if tracked_person.size != 0:
            # add averages of all tracked points as pixel x and y
            self.tracked_objects["tracked_person"].pixel_x = np.mean(
                tracked_person[:, 0], axis=0
            )
            self.tracked_objects["tracked_person"].pixel_y = np.mean(
                tracked_person[:, 1], axis=0
            )
            self.tracked_objects["tracked_person"].extra["landmarks"] = tracked_person
        else:
            self.tracked_objects["tracked_person"].pixel_x = None
            self.tracked_objects["tracked_person"].pixel_y = None
            self.tracked_objects["tracked_person"].extra["landmarks"] = np.full(
                (1, YOLOModelInfo.num_tracked_points, 2), np.nan
            )


if __name__ == "__main__":
    YOLOPoseTracker().demo()



import cv2

from skellytracker.trackers.bright_point_tracker.brightest_point_tracker import BrightestPointTracker
from skellytracker.trackers.charuco_tracker.charuco_tracker import CharucoTracker
from skellytracker.trackers.mediapipe_tracker.mediapipe_holistic_tracker import MediapipeHolisticTracker
from skellytracker.trackers.segment_anything_tracker.segment_anything_tracker import SAMTracker
from skellytracker.trackers.yolo_tracker.yolo_tracker import YOLOPoseTracker
from skellytracker.trackers.yolo_object_tracker.yolo_object_tracker import YOLOObjectTracker

if __name__ == "__main__":

    demo_tracker = "yolo_object_tracker"


    if demo_tracker == "brightest_point_tracker":
        BrightestPointTracker().demo()

    elif demo_tracker == "charuco_tracker":
        CharucoTracker(squaresX=7,
                       squaresY=5,
                       dictionary=cv2.aruco.getPredefinedDictionary(cv2.aruco.DICT_4X4_250)).demo()

    elif demo_tracker == "mediapipe_holistic_tracker":
        MediapipeHolisticTracker(model_complexity=2,
                                 min_detection_confidence=0.5,
                                 min_tracking_confidence=0.5,
                                 static_image_mode=False,
                                 smooth_landmarks=True).demo()

    elif demo_tracker == "yolo_tracker":
        YOLOPoseTracker(model_size="high_res").demo()
    elif demo_tracker == "SAM_tracker":
        SAMTracker().demo()
    elif demo_tracker == "yolo_object_tracker":
        YOLOObjectTracker(model_size="medium").demo()
